Imports I8123W_Net

Public Class Form1

    Dim SLOT As Byte
    Dim BAUD As Byte
    Dim NODE As Byte
    Dim STA As Byte

    Dim BlockMode As Byte = 1

    Dim STA_Str As String = ""
    Dim NMTErr As String = ""
    Dim EMCY As String = ""

    Dim DelayTime As UInt16
    Dim ret As UInt16
    Dim errstr As String

    Dim MyNMTErrISR As I8123W.NMTErrISR
    Dim MyEMCYISR As I8123W.EMCYISR

    Sub UserNMTErrISR()

        Dim board As Byte, node As Byte, mode As Byte
        Dim tempstr As String = ""

        If (I8123W.I8123_GetNMTError(board, node, mode) = 0) Then

            If (mode = I8123W.CPM_Node_Guarding_Event) Then

                tempstr = "NMTErr: Board " + board.ToString("D") + ", Node " + node.ToString("D") + " Guarding Fail"

            ElseIf (mode = I8123W.CPM_Heartbeat_Event) Then

                tempstr = "NMTErr: Board " + board.ToString("D") + ", Node " + node.ToString("D") + " Heartbeat Error"
            End If
        End If

        NMTErr = tempstr

    End Sub

    Sub UserEMCYISR()

        Dim board As Byte, node As Byte
        Dim data(8) As Byte
        Dim tempstr As String = ""

        If (I8123W.I8123_GetEMCYData(board, node, data) = 0) Then

            Dim i As UShort

            tempstr = "EMCY: Board " + board.ToString("D") + ", Node " + node.ToString("D") + " :"

            For i = 0 To 7
                tempstr = tempstr + data(i).ToString("X2") + " "
            Next

        End If

        EMCY = tempstr

    End Sub

    Private Sub Form1_Closed(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        Call I8123W.I8123_ShutdownMaster(SLOT)
    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        SLOT = 0
        BAUD = 0
        NODE = 1

        STA = &H1
        STA_Str = "Operation mode"

        Combo_Slot.Enabled = True
        Combo_Baud.Enabled = True
        Combo_Node.Enabled = False

        Combo_Slot.SelectedIndex = 0
        Combo_Baud.SelectedIndex = 0
        Combo_Node.SelectedIndex = 0
        Combo_State.SelectedIndex = 0

        Btn_Initial.Enabled = True
        Btn_Shutdown.Enabled = False
        Btn_AddNode.Enabled = False
        Btn_Remove.Enabled = False

        Pal_NMT.Enabled = False
    End Sub

    Private Sub Combo_Slot_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Combo_Slot.SelectedIndexChanged
        SLOT = Combo_Slot.SelectedIndex
    End Sub

    Private Sub Combo_Baud_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Combo_Baud.SelectedIndexChanged

        BAUD = Combo_Baud.SelectedIndex

        Select Case (BAUD)
            Case 0
                DelayTime = 15
            Case 1
                DelayTime = 7
            Case 2
                DelayTime = 3
            Case 3
            Case 4
                DelayTime = 1
            Case 5
            Case 6
            Case 7
                DelayTime = 0
        End Select
    End Sub

    Private Sub Combo_Node_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Combo_Node.SelectedIndexChanged

        NODE = Combo_Node.SelectedIndex + 1
    End Sub

    Private Sub Combo_State_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Combo_State.SelectedIndexChanged
        Select Case (Combo_State.SelectedIndex)
            Case 0
                STA = &H1   'Operation mode
                STA_Str = "Operation mode"
            Case 1
                STA = &H2   'Stop mode
                STA_Str = "Stop mode"
            Case 2
                STA = &H80  'Pre-Operation mode
                STA_Str = "Pre-Operation mode"
            Case 3
                STA = &H81  'Reset Node
                STA_Str = "Reset Node"
            Case 4
                STA = &H82  'Reset Communication
                STA_Str = "Reset Communication"
        End Select
    End Sub

    Private Sub Btn_SetState_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_SetState.Click

        ret = I8123W.I8123_NMTChangeState(SLOT, NODE, STA, BlockMode)
        If ret <> 0 Then
            errstr = "Change Slave State Error, = " + ret.ToString("D")
            MessageBox.Show(errstr)
        Else
            errstr = "Change Slave State to" + STA_Str + " OK"
            List_EMCY.Items.Add(errstr)
        End If
    End Sub

    Private Sub Btn_Initial_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Initial.Click

        ret = I8123W.I8123_InitMaster(SLOT, 0, BAUD, BlockMode)
        If ret <> 0 Then
            errstr = "Initial Error, = " + ret.ToString("D")
            MessageBox.Show(errstr)
        Else
            MyNMTErrISR = New I8123W.NMTErrISR(AddressOf UserNMTErrISR)
            MyEMCYISR = New I8123W.EMCYISR(AddressOf UserEMCYISR)

            Call I8123W.I8123_InstallNMTErrISR(SLOT, MyNMTErrISR)
            Call I8123W.I8123_InstallEMCYISR(SLOT, MyEMCYISR)

            Combo_Slot.Enabled = False
            Combo_Baud.Enabled = False
            Combo_Node.Enabled = True

            Btn_Initial.Enabled = False
            Btn_Initial.Visible = False
            Btn_Shutdown.Enabled = True

            Btn_AddNode.Enabled = True
            Btn_AddNode.Visible = True

            Timer1.Enabled = True
        End If
    End Sub

    Private Sub Btn_Shutdown_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Shutdown.Click

        ret = I8123W.I8123_ShutdownMaster(SLOT)
        If ret <> 0 Then
            errstr = "Shutdown Error, = " + ret.ToString("D")
            MessageBox.Show(errstr)
        Else
            Call I8123W.I8123_RemoveNMTErrISR(SLOT)
            Call I8123W.I8123_RemoveEMCYISR(SLOT)

            Combo_Slot.Enabled = True
            Combo_Baud.Enabled = True
            Combo_Node.Enabled = False

            Btn_Initial.Enabled = True
            Btn_Initial.Visible = True
            Btn_Shutdown.Enabled = False

            Btn_AddNode.Enabled = False
            Btn_AddNode.Visible = True
            Btn_Remove.Enabled = False

            Pal_NMT.Enabled = False

            Timer1.Enabled = False

            List_EMCY.Items.Clear()
        End If
    End Sub

    Private Sub Btn_AddNode_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_AddNode.Click

        ret = I8123W.I8123_AddNode(SLOT, NODE, 1, DelayTime, 200, BlockMode)
        If ret <> 0 Then
            errstr = "AddNode Error, = " + ret.ToString("D")
            MessageBox.Show(errstr)
        Else
            Btn_AddNode.Enabled = False
            Btn_AddNode.Visible = False
            Btn_Remove.Enabled = True
            Combo_Node.Enabled = False

            Pal_NMT.Enabled = True
        End If
    End Sub

    Private Sub Btn_Remove_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Remove.Click

        ret = I8123W.I8123_RemoveNode(SLOT, NODE, BlockMode)
        If ret <> 0 Then
            errstr = "RemoveNode Error, = " + ret.ToString("D")
            MessageBox.Show(errstr)
        Else
            Btn_Remove.Enabled = False
            Btn_AddNode.Enabled = True
            Btn_AddNode.Visible = True
            Combo_Node.Enabled = True

            Call List_EMCY.Items.Clear()

            Pal_NMT.Enabled = False
        End If
    End Sub

    Private Sub Btn_Clear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Clear.Click
        Call List_EMCY.Items.Clear()
    End Sub

    Private Sub Check_Is_Dec(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles Text_GuardTime.KeyPress, Text_LifeTime.KeyPress, Text_Heartbeat.KeyPress, Text_Consumer.KeyPress
        If ((("1234567890".IndexOf(e.KeyChar) <> -1) Or (Convert.ToInt16(e.KeyChar) = 8)) = False) Then e.Handled = True
    End Sub

    Private Sub Btn_SetGuard_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_SetGuard.Click
        Dim GuardTime As UInt16
        Dim LifeTime As Byte

        GuardTime = Val(Text_GuardTime.Text)
        LifeTime = Val(Text_LifeTime.Text)

        ret = I8123W.I8123_NMTGuarding(SLOT, NODE, GuardTime, LifeTime, BlockMode)
        If ret <> 0 Then
            errstr = "Set Guarding Error, = " + ret.ToString("D")
            MessageBox.Show(errstr)
        Else
            errstr = "Set Guarding OK"
            List_EMCY.Items.Add(errstr)
        End If
    End Sub

    Private Sub Btn_SetHeartbeat_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_SetHeartbeat.Click
        Dim Heartbeat As UInt16, Consumer As UInt16

        Heartbeat = Val(Text_Heartbeat.Text)
        Consumer = Val(Text_Consumer.Text)

        ret = I8123W.I8123_NMTHeartbeat(SLOT, NODE, Heartbeat, Consumer, BlockMode)
        If ret <> 0 Then
            errstr = "Set Heartbeat Error, = " + ret.ToString("D")
            MessageBox.Show(errstr)
        Else
            errstr = "Set Heartbeat OK"
            List_EMCY.Items.Add(errstr)
        End If
    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick

        If EMCY <> "" Then
            List_EMCY.Items.Add(EMCY)
            EMCY = ""
        End If

        If NMTErr <> "" Then
            List_EMCY.Items.Add(NMTErr)
            NMTErr = ""
        End If

    End Sub
End Class
