Imports XPacNET
Imports I8123W_Net

Public Class Form1

    Dim SLOT As Byte
    Dim BAUD As Byte
    Dim NODE As Byte

    Dim BlockMode As Byte = 1

    Dim ret As UInt16
    Dim errstr As String

    Const DI_MODULE_SLOT = 2
    Const DI_TOTAL_CH = 16

    Const DO_MODULE_SLOT = 1
    Const DO_TOTAL_CH = 16

    Dim RSDOISR As I8123W.ReadSDOISR
    Dim WSDOISR As I8123W.WriteSDOISR
    Dim RXPDOISR As I8123W.RxPDOISR
    Dim REMOTEPDOISR As I8123W.RemotePDOISR

    Dim ISR_Str As String = ""
    Dim HPORT As System.IntPtr

    Sub UserRSDOISR()
        Dim Ret As UShort, Index As UShort
        Dim SlotNo As Byte, SubIndex As Byte, RLen As Byte
        Dim RData(4) As Byte
        Dim Abort As UInteger

        Ret = I8123W.I8123_GetMasterReadSDOEvent(SlotNo, Index, SubIndex)
        If Ret = 0 Then
            Select Case (Index)
                Case &H6000 ' Read DI
                    Dim DI_Value As UInteger = 0

                    Select Case (SubIndex)
                        Case &H0
                            ' DI entry number
                            RLen = 1
                            RData(0) = &H2
                        Case &H1
                            RLen = 1
                            ' Read DI value of ch0 ~ ch7
                            XPac.pac_ReadDI(HPORT, DI_MODULE_SLOT, DI_TOTAL_CH, DI_Value)
                            RData(0) = Convert.ToByte(DI_Value And &HFF)
                        Case &H2
                            RLen = 1

                            ' Read DI value of ch8 ~ ch15
                            XPac.pac_ReadDI(HPORT, DI_MODULE_SLOT, DI_TOTAL_CH, DI_Value)
                            RData(0) = Convert.ToByte((DI_Value >> 8) And &HFF)
                        Case Else
                            Abort = &H6090011

                            ' Abort: Sub-index does not exist.
                            RLen = 4
                            RData = BitConverter.GetBytes(Abort)
                    End Select
                Case &H6200 ' Read DO
                    Dim DO_Value As Integer = 0

                    Select Case (SubIndex)
                        Case &H0
                            ' DO entry number
                            RLen = 1
                            RData(0) = &H2
                        Case &H1
                            RLen = 1

                            ' Read DO value of ch0 ~ ch7
                            XPac.pac_ReadDO(HPORT, DO_MODULE_SLOT, DO_TOTAL_CH, DO_Value)
                            RData(0) = Convert.ToByte(DO_Value And &HFF)
                        Case &H2
                            RLen = 1

                            ' Read DO value of ch8 ~ ch15
                            XPac.pac_ReadDO(HPORT, DO_MODULE_SLOT, DO_TOTAL_CH, DO_Value)
                            RData(0) = Convert.ToByte((DO_Value >> 8) And &HFF)
                        Case Else
                            Abort = &H6090011

                            ' Abort: Sub-index does not exist.
                            RLen = 4
                            RData = BitConverter.GetBytes(Abort)
                    End Select
                Case Else
                    Abort = &H6020000

                    ' Abort: Object does not exist.
                    RLen = 4
                    RData = BitConverter.GetBytes(Abort)
            End Select

            Dim i As UShort
            Dim TempStr As String

            If (Abort = 0) Then
                I8123W.I8123_ResponseMasterSDO(SlotNo, 0, Index, SubIndex, RLen, RData)
                TempStr = "SDO Read"

            Else
                I8123W.I8123_SDOAbortTransmit(SlotNo, NODE, Index, SubIndex, Abort, BlockMode)
                TempStr = "SDO Abort"
            End If

            TempStr = TempStr + " " + Index.ToString("X4") + "-" + SubIndex.ToString("X2") + ":"

            If RLen <> 0 Then
                For i = 0 To RLen - 1

                    TempStr = TempStr + " " + RData(i).ToString("X2")
                Next i
            End If

            ISR_Str = TempStr

        End If
    End Sub

    Sub UserWSDOISR()
        Dim Ret As UShort, Index As UShort
        Dim SlotNo As Byte, SubIndex As Byte, WLen As Byte
        Dim WData(4) As Byte
        Dim Abort As UInteger

        Ret = I8123W.I8123_GetMasterWriteSDOEvent(SlotNo, Index, SubIndex, WLen, WData)
        If Ret = 0 Then
            Select Case (Index)
                Case &H6200 ' Write DO
                    Dim DO_Value As UInteger = 0

                    If (WLen <> 1) Then
                        Abort = &H6070010

                        ' Abort: Data type does not match, length of service parameter does not match
                        WLen = 4
                        WData = BitConverter.GetBytes(Abort)
                    End If

                    Select Case (SubIndex)
                        Case &H0
                            Abort = &H6010002

                            ' Abort: Attempt to write a read only object
                            WLen = 4
                            WData = BitConverter.GetBytes(Abort)
                        Case &H1
                            Dim temp_value As UInteger
                            ' Get original DO value
                            XPac.pac_ReadDO(HPORT, DO_MODULE_SLOT, DO_TOTAL_CH, DO_Value)

                            ' Change DO value of ch0 ~ ch7
                            temp_value = WData(0)
                            DO_Value = (DO_Value And &HFFFFFF00) Or (temp_value And &HFF00)
                            XPac.pac_WriteDO(HPORT, DO_MODULE_SLOT, DO_TOTAL_CH, DO_Value)

                            ' Write OK
                        Case &H2
                            Dim temp_value As UInteger
                            ' Get original DO value
                            XPac.pac_ReadDO(HPORT, DO_MODULE_SLOT, DO_TOTAL_CH, DO_Value)

                            ' Change DO value of ch8 ~ ch15
                            temp_value = WData(0)
                            DO_Value = (DO_Value And &HFFFF00FF) Or ((temp_value << 8) And &HFF00)
                            XPac.pac_WriteDO(HPORT, DO_MODULE_SLOT, DO_TOTAL_CH, DO_Value)

                            ' Write SDO OK
                        Case Else
                            Abort = &H6090011

                            ' Abort: Sub-index does not exist.
                            WLen = 4
                            WData = BitConverter.GetBytes(Abort)
                    End Select
                Case Else
                    Abort = &H6090011

                    ' Abort: Object does not exist.
                    WLen = 4
                    WData = BitConverter.GetBytes(Abort)
            End Select

            Dim i As UShort
            Dim TempStr As String

            If (Abort = 0) Then
                I8123W.I8123_ResponseMasterSDO(SlotNo, 0, Index, SubIndex, WLen, WData)
                TempStr = "SDO Write " + Index.ToString("X4") + "-" + SubIndex.ToString("X2") + ":  OK"

            Else
                I8123W.I8123_SDOAbortTransmit(SlotNo, NODE, Index, SubIndex, Abort, BlockMode)
                TempStr = "SDO Abort " + Index.ToString("X4") + "-" + SubIndex.ToString("X2") + ":"

                If WLen <> 0 Then
                    For i = 0 To WLen - 1

                        TempStr = TempStr + " " + WData(i).ToString("X2")
                    Next i
                End If
            End If

            ISR_Str = TempStr

        End If
    End Sub

    Sub UserRXPDOISR()

        Dim Ret As UShort
        Dim SlotNo As Byte, WLen As Byte
        Dim CobId As UShort, i As UShort
        Dim WData(8) As Byte
        Dim TempStr As String

        Ret = I8123W.I8123_GetMasterRxPDOEvent(SlotNo, CobId, WLen, WData)
        If Ret = 0 Then
            If (CobId = &H200 + NODE) Then
                If (WLen >= 2) Then
                    Dim DO_Value As UInteger

                    ' Output DO value of ch0 ~ ch15
                    DO_Value = BitConverter.ToUInt32(WData, 0)
                    XPac.pac_WriteDO(HPORT, DO_MODULE_SLOT, DO_TOTAL_CH, DO_Value)

                    TempStr = "PDO " + CobId.ToString("X3") + " Output:"
                    For i = 0 To (WLen - 1)
                        TempStr = TempStr + " " + WData(i).ToString("X2")
                    Next i

                    ISR_Str = TempStr
                End If
            End If
        End If
    End Sub

    Sub UserREMOTEPDOISR()

        Dim Ret As UShort
        Dim BoardNo As Byte, RLen As Byte
        Dim CobId As UShort, i As UShort
        Dim RData(8) As Byte
        Dim TempStr As String

        Ret = I8123W.I8123_GetMasterRemotePDOEvent(BoardNo, CobId)
        If Ret = 0 Then

            If (CobId = &H180 + NODE) Then
                Dim DI_Value As UInteger = 0

                ' Read DI value of ch0 ~ ch15
                XPac.pac_ReadDI(HPORT, DI_MODULE_SLOT, DI_TOTAL_CH, DI_Value)

                TempStr = "PDO " + CobId.ToString("X3") + " Remote:"

                If RLen <> 0 Then
                    For i = 0 To RLen - 1
                        TempStr = TempStr + " " + RData(i).ToString("X2")
                    Next i
                End If

                ISR_Str = TempStr

            End If
        End If
    End Sub

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        SLOT = 0
        BAUD = 0
        NODE = 1

        Combo_Slot.Enabled = True
        Combo_Baud.Enabled = True
        Combo_Node.Enabled = True

        Combo_Slot.SelectedIndex = 0
        Combo_Baud.SelectedIndex = 0
        Combo_Node.SelectedIndex = 0

        Btn_Clear.Enabled = False

        HPORT = XPac.uart_Open("COM1:,115200,N,8,1")

    End Sub

    Private Sub Form1_Closed(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Closed
        Call I8123W.I8123_ShutdownMaster(SLOT)
    End Sub

    Private Sub Combo_Slot_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Combo_Slot.SelectedIndexChanged
        SLOT = Combo_Slot.SelectedIndex
    End Sub

    Private Sub Combo_Baud_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Combo_Baud.SelectedIndexChanged
        BAUD = Combo_Baud.SelectedIndex
    End Sub

    Private Sub Combo_Node_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Combo_Node.SelectedIndexChanged
        NODE = Combo_Node.SelectedIndex + 1
    End Sub

    Private Sub Btn_Initial_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Initial.Click

        Dim Entry_Data(8) As UInt32

        ret = I8123W.I8123_InitMaster(SLOT, NODE, BAUD, BlockMode)
        If ret <> 0 Then
            errstr = "Initial Error, = " + ret.ToString("D")
            MessageBox.Show(errstr)
        Else
            Combo_Slot.Enabled = False
            Combo_Baud.Enabled = False
            Combo_Node.Enabled = False

            Btn_Initial.Enabled = False
            Btn_Initial.Visible = False
            Btn_Shutdown.Enabled = True

            ' Install SDO read ISR
            RSDOISR = New I8123W.ReadSDOISR(AddressOf UserRSDOISR)
            ret = I8123W.I8123_InstallReadSDOISR(SLOT, RSDOISR)
            If ret <> 0 Then
                errstr = "Install Read SDO ISR Error, = " + ret.ToString("D")
                MessageBox.Show(errstr)
            End If

            ' Initial SDO write ISR
            WSDOISR = New I8123W.WriteSDOISR(AddressOf UserWSDOISR)
            ret = I8123W.I8123_InstallWriteSDOISR(SLOT, WSDOISR)
            If ret <> 0 Then
                errstr = "Install Write SDO ISR Error, = " + ret.ToString("D")
                MessageBox.Show(errstr)
            End If

            ' Install PDO output ISR
            RXPDOISR = New I8123W.RxPDOISR(AddressOf UserRXPDOISR)
            ret = I8123W.I8123_InstallRxPDOISR(SLOT, RXPDOISR)
            If ret <> 0 Then
                errstr = "Install output PDO ISR Error, = " + ret.ToString("D")
                MessageBox.Show(errstr)
            End If

            ' The "EntryUse" parameter and "EntryData" parameter are useless
            ' when the PDO had been installed to master. 
            Call I8123W.I8123_InstallPDO_List(SLOT, NODE, &H200 + NODE, 0, 1, 255, 0, 0, Entry_Data, BlockMode)

            ' Install PDO remote ISR
            REMOTEPDOISR = New I8123W.RemotePDOISR(AddressOf UserREMOTEPDOISR)
            ret = I8123W.I8123_InstallRemotePDOISR(SLOT, REMOTEPDOISR)
            If ret <> 0 Then
                errstr = "Install remote PDO ISR Error, = " + ret.ToString("D")
                MessageBox.Show(errstr)
            End If

            ' The "EntryUse" parameter and "EntryData" parameter are useless
            ' when the PDO had been installed to master.
            Call I8123W.I8123_InstallPDO_List(SLOT, NODE, &H180 + NODE, 1, 1, 255, 0, 0, Entry_Data, BlockMode)

            Timer1.Enabled = True

        End If
    End Sub

    Private Sub Btn_Shutdown_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Shutdown.Click

        ret = I8123W.I8123_ShutdownMaster(SLOT)
        If ret <> 0 Then
            errstr = "Shutdown Error, = " + ret.ToString("D")
            MessageBox.Show(errstr)
        Else
            Combo_Slot.Enabled = True
            Combo_Baud.Enabled = True
            Combo_Node.Enabled = True

            Btn_Initial.Enabled = True
            Btn_Initial.Visible = True
            Btn_Shutdown.Enabled = False

            Combo_Node.SelectedIndex = 0

            List_Monitor.Items.Clear()

            Timer1.Enabled = False
        End If
    End Sub

    Private Sub Btn_Clear_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Btn_Clear.Click
        List_Monitor.Items.Clear()
    End Sub

    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick

        If (ISR_Str <> "") Then
            List_Monitor.Items.Add(ISR_Str)
            ISR_Str = ""
        End If

    End Sub
End Class
